#ifndef GUIMESSAGEDIALOG_H
#define GUIMESSAGEDIALOG_H

#include <QDialog>
#include <QList>
namespace Ui {
class GUIMessageDialog;
}

class GUIMessageDialog : public QDialog
{
    Q_OBJECT

public:
    explicit GUIMessageDialog(QWidget *parent = nullptr);
    ~GUIMessageDialog();
    void showMessage(QString msg);
    void hideMessage();
    void showExitButton();
    void hideExitButton();
    void startLoading();
    void stopLoading();
    void setOpacity(double);
protected:
    void timerEvent(QTimerEvent* event) override ;
private:
    Ui::GUIMessageDialog *ui;
    QList<QString> frame;
    int frameIndex=0;
    int timerID = -1;
};

#endif // GUIMESSAGEDIALOG_H
